/*	Serhat Sevki Dincer, jfcgaussATgmail, 12apr2006
	Random Number Generation Add-In for OpenOffice.org Calc

	This program can be used, modified and/or (re)distributed under
	the terms of LGPL. See www.gnu.org for info about LGPL.

	This program has NO warranty.
	OpenOffice.org SDK has been used while writing this program.
*/

#include <cppuhelper/implbase4.hxx> //4-parameter template will be used
#include <cppuhelper/factory.hxx>
#include <cppuhelper/implementationentry.hxx>

#include <com/sun/star/sheet/XAddIn.hpp>
#include <com/sun/star/lang/XServiceName.hpp>
#include <com/sun/star/lang/XServiceInfo.hpp>
#include <org/openoffice/sheet/addin/XRNG.hpp>

#include <cstdlib>
#include <cmath>

using namespace ::rtl;
using namespace ::com::sun::star;
using namespace ::com::sun::star::uno;

namespace _RNG_impl_
{
	class RNG_impl : public ::cppu::WeakImplHelper4< ::org::openoffice::sheet::addin::XRNG,
		sheet::XAddIn, lang::XServiceName, lang::XServiceInfo > //4-parameter template
	{
	//Locale
		lang::Locale locale;

	public:
	//XRNG
		double SAL_CALL expo( double m ) throw (RuntimeException); //expo(mean)

	//XAddIn
		OUString SAL_CALL getProgrammaticFuntionName( const OUString& aDisplayName ) throw (RuntimeException);
		OUString SAL_CALL getDisplayFunctionName( const OUString& aProgrammaticName ) throw (RuntimeException);
		OUString SAL_CALL getFunctionDescription( const OUString& aProgrammaticName ) throw (RuntimeException);
		OUString SAL_CALL getDisplayArgumentName( const OUString& aProgrammaticName, ::sal_Int32 nArgument ) throw (RuntimeException);
		OUString SAL_CALL getArgumentDescription( const OUString& aProgrammaticName, ::sal_Int32 nArgument ) throw (RuntimeException);
		OUString SAL_CALL getProgrammaticCategoryName( const OUString& aProgrammaticName ) throw (RuntimeException);
		OUString SAL_CALL getDisplayCategoryName( const OUString& aProgrammaticName ) throw (RuntimeException);

	//XServiceName
		OUString SAL_CALL getServiceName(  ) throw (RuntimeException);

	//XServiceInfo
		OUString SAL_CALL getImplementationName(  ) throw (RuntimeException);
		::sal_Bool SAL_CALL supportsService( const OUString& ServiceName ) throw (RuntimeException);
		Sequence< OUString > SAL_CALL getSupportedServiceNames(  ) throw (RuntimeException);

	//XLocalizable
		void SAL_CALL setLocale( const lang::Locale& eLocale ) throw (RuntimeException);
		lang::Locale SAL_CALL getLocale(  ) throw (RuntimeException);
	};

//XRNG
	//expo(mean)
	double RNG_impl::expo( double m ) throw (RuntimeException)
	{
		return -m * log( (double)(1+(unsigned int)rand()) / (2+(unsigned int)RAND_MAX) );
	}


	#define _serviceName_ "org.openoffice.sheet.addin.RNG"

	static const sal_Char *_serviceName = _serviceName_;
	static const sal_Char *_functionDisplayName = "EXPO";
	static const sal_Char *_functionName = "expo";
	static const sal_Char *_functionDescription = "Returns a random # from an Exponential(m) distribution";

//XAddIn
	OUString RNG_impl::getProgrammaticFuntionName( const OUString& aDisplayName ) throw (RuntimeException)
	{
		if(aDisplayName.equalsAscii(_functionDisplayName))
		{
			OString os(_functionName);
			return OStringToOUString(os, RTL_TEXTENCODING_ASCII_US);
		}
		return OUString(RTL_CONSTASCII_USTRINGPARAM(""));
	}

	OUString RNG_impl::getDisplayFunctionName( const OUString& aProgrammaticName ) throw (RuntimeException)
	{
		if(aProgrammaticName.equalsAscii(_functionName))
		{
			OString os(_functionDisplayName);
			return OStringToOUString(os, RTL_TEXTENCODING_ASCII_US);
		}
		return OUString(RTL_CONSTASCII_USTRINGPARAM(""));
	}

	OUString RNG_impl::getFunctionDescription( const OUString& aProgrammaticName ) throw (RuntimeException)
	{
		if(aProgrammaticName.equalsAscii(_functionName))
		{
			OString os(_functionDescription);
			return OStringToOUString(os, RTL_TEXTENCODING_ASCII_US);
		}
		return OUString(RTL_CONSTASCII_USTRINGPARAM(""));
	}

	OUString RNG_impl::getDisplayArgumentName( const OUString& aProgrammaticName, ::sal_Int32 nArgument ) throw (RuntimeException)
	{
		if(aProgrammaticName.equalsAscii(_functionName) && nArgument==0) return OUString(RTL_CONSTASCII_USTRINGPARAM("m"));
		return OUString(RTL_CONSTASCII_USTRINGPARAM(""));
	}

	OUString RNG_impl::getArgumentDescription( const OUString& aProgrammaticName, ::sal_Int32 nArgument ) throw (RuntimeException)
	{
		if(aProgrammaticName.equalsAscii(_functionName) && nArgument==0) return OUString(RTL_CONSTASCII_USTRINGPARAM("mean"));
		return OUString(RTL_CONSTASCII_USTRINGPARAM(""));
	}

	OUString RNG_impl::getProgrammaticCategoryName( const OUString& aProgrammaticName ) throw (RuntimeException)
	{
		return OUString(RTL_CONSTASCII_USTRINGPARAM("RNG"));
	}

	OUString RNG_impl::getDisplayCategoryName( const OUString& aProgrammaticName ) throw (RuntimeException)
	{
		return OUString(RTL_CONSTASCII_USTRINGPARAM("RNG"));
	}

//XServiceName
	OUString RNG_impl::getServiceName(  ) throw (RuntimeException)
	{
		return OUString(_serviceName, sizeof(_serviceName_)-1, RTL_TEXTENCODING_ASCII_US);
	}

//XServiceInfo
	static OUString getImplementationName_RNG_impl() throw (RuntimeException)
	{
		return OUString(RTL_CONSTASCII_USTRINGPARAM("org.openoffice.sheet.addin.RNG_impl.RNG"));
	}

	OUString RNG_impl::getImplementationName() throw (RuntimeException)
	{
		return getImplementationName_RNG_impl();
	}

	::sal_Bool RNG_impl::supportsService( OUString const & serviceName ) throw (RuntimeException)
	{
		return serviceName.equalsAsciiL(_serviceName, sizeof(_serviceName_)-1);
	}

	static Sequence< OUString > getSupportedServiceNames_RNG_impl() throw (RuntimeException)
	{
		Sequence< OUString > name(1);
		name[0] = OUString(_serviceName, sizeof(_serviceName_)-1, RTL_TEXTENCODING_ASCII_US);
		return name;
	}

	Sequence< OUString > RNG_impl::getSupportedServiceNames() throw (RuntimeException)
	{
		return getSupportedServiceNames_RNG_impl();
	}

//XLocalizable
	void RNG_impl::setLocale( const lang::Locale& eLocale ) throw (RuntimeException)
	{
		locale = eLocale;
	}

	lang::Locale RNG_impl::getLocale(  ) throw (RuntimeException)
	{
		return locale;
	}


	static Reference< XInterface > SAL_CALL create_RNG_impl( Reference< XComponentContext > const & xContext ) SAL_THROW( () )
	{
		return static_cast< ::cppu::OWeakObject * > ( new RNG_impl );
	}

	static struct ::cppu::ImplementationEntry s_component_entries[] =
	{
		{ create_RNG_impl, getImplementationName_RNG_impl,
			getSupportedServiceNames_RNG_impl, ::cppu::createSingleComponentFactory, 0, 0 }
		,
		{ 0, 0, 0, 0, 0, 0 }
	};
}

extern "C"
{
	void SAL_CALL component_getImplementationEnvironment( sal_Char const ** ppEnvTypeName, uno_Environment ** ppEnv )
	{
		*ppEnvTypeName = CPPU_CURRENT_LANGUAGE_BINDING_NAME;
	}

	sal_Bool SAL_CALL component_writeInfo( lang::XMultiServiceFactory * xMgr, registry::XRegistryKey * xRegistry )
	{
		return ::cppu::component_writeInfoHelper( xMgr, xRegistry, ::_RNG_impl_::s_component_entries );
	}

	void * SAL_CALL component_getFactory( sal_Char const * implName,
		lang::XMultiServiceFactory * xMgr, registry::XRegistryKey * xRegistry )
	{
		return ::cppu::component_getFactoryHelper(implName, xMgr, xRegistry, ::_RNG_impl_::s_component_entries );
	}
}
